#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 01/02/2002
#################################################################################
#
# Configure Puppeteer - Script part of Webchat designed to configure Puppeteers
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);                                    # MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handlingS
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash

my %config;							# Configuration tag
my %login;							# Login handler

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch hash
	$database->GetHashConfiguration(\%config);
	$database->GetHashLogin(\%login);

	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login    = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Print title
		print "$config{'TagTitle'}\n<BR>\n";

		# Puppet activity handling
		if (defined($param->{'PUPPET'}))
		{ ActivityHandler($database, $cgi, $param, \%config); }

		# Chat preferences handling
		if (defined($param->{'PREFS'}))
		{ ChatHandler($database, $param, \%login); }

		# Information handling
		if (defined($param->{'OLDPASS'}) || defined($param->{'EMAIL'}))
		{ InformationHandler($database, $cgi, $param, \%config); }


		# Initial screen	
		InitialDisplay($database, $cgi, $param, \%config, \%login);
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Activity Handler
#
# Will handle the activation de-activation of puppets as designated by the user
# this is to reduce the long list of puppets for selection when some are rarely
# used if ever.

sub ActivityHandler						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration tags	

	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $puppet;						# Puppet to handle
	my $active;						# Activity flag


	#####################
	# Program area	

	# Find appropraite values
	$active = (defined($param->{$config->{'TxtActive'}})) ? undef : time;
	$puppet = (defined($param->{$config->{'TxtActive'}})) ? $param->{'IPUPPET'}: $param->{'APUPPET'};

	# Update tables
	$data->do("UPDATE Puppet
		SET PuppetActive=?
		WHERE PuppetName=?
		  AND PuppeteerLogin=?", {},
	 $active,
	 $puppet,
	 $param->{'USER'});

	# Cleanup
	$param->Cleanup(
		'PUPPET',
		'APUPPET',
		'IPUPPET',
		$config->{'TxtActive'},
		$config->{'TxtInactive'}
	 );
}


#####################
# Chat Prefereces Handler
#
# Takes care of the database updates when one wants to change chat preferences
# this is done by figuring out what the options where.

sub ChatHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $login    = shift;					# Configuration tags	

	my $data     = $database->{'HANDLE'};			# Database handle

	my $option;						# Options handle
	my $rjava;						# Formated javascript
	my $rchat;						# Formatted chat preferences
	my $rformat;						# Formatted format options


	#####################
	# Program area

	# Verify options received
	$option  = new Ethereal::Option();
	$rjava   = $option->FindJavascript($login{'OptJavascript'}, $param->{'CJAVA'});
	$rchat   = $option->FindChatPrefs($login{'OptChatPref'}, $param->{'CCHAT'});
	$rformat = $option->FindFormatting($login{'OptFormat'}, $param->{'CFORMAT'});


	# Javascript options
	if ($rjava ne 'default')
	{
		# Update database entry
		$data->do("UPDATE Puppeteer
			SET   PuppeteerJavascript=?
			WHERE PuppeteerLogin=?", {},

		 $rjava,
		 $param->{'USER'});
	}

	# Chatting options
	if ($rchat ne 'default')
	{
		# Update database entry
		$data->do("UPDATE Puppeteer
			SET   PuppeteerChatPref=?
			WHERE PuppeteerLogin=?", {},

		 $rchat,
		 $param->{'USER'});
	}

	# Formatting options
	if ($rformat ne 'default')
	{
		# Update database entry
		$data->do("UPDATE Puppeteer
			SET   PuppeteerFormatting=?
			WHERE PuppeteerLogin=?", {},

		 $rformat,
		 $param->{'USER'});
	}
}

#####################
# Check Data
#  
# Arbitrary sub-routine designed to be used to check validity of data.  Made
# to hide ugliness of checks also centralize adapting of checks.

sub CheckData
{
	#####################
	# Data members
	my $value    = shift;					# Value to check 
	my $choice   = shift;					# Choice of the matter
	my $database = shift;					# Possible database handle
	my $user     = shift;					# User


	#####################
	# Program area

	# Password Check
	if ($choice == 0)
	{
		# Retreive Password
		my ($crypt) = $database->DataGet("SELECT PuppeteerPassword
			FROM  Puppeteer
			WHERE PuppeteerLogin=?", $user);	


		# Verify existence of user and validity of password
		if ((defined($crypt)) && ($crypt eq crypt($value, $crypt)))
		{ return 1; }					# Return true
	}

	# Length
	elsif ($choice == 1)
	{
		# Lenght of a minumum of 3 characters
		if (length($value) > 3)
		{ return 1; }					# Return true
	}

	# E-Mail
	elsif ($choice == 2)
	{
		# Verify for legality
		if ($value =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s)
		{ return 1; }					# Return true
	}

	return 0; 						# False if all else fails
}

# Alias for Password validity
sub CheckDataPassword
{ CheckData($_[2], 0, $_[0], $_[1]); }

# Alias for Lenght
sub CheckDataLenght
{ CheckData(shift, 1); }

# Alias for E-Eail validity
sub CheckDataEMail
{ CheckData(shift, 2); }


#####################
# Information Handler
#
# Will take the parameters and depending on information received can change the
# passwords and email.  The rest is considered non-essential.

sub InformationHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash
	
	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $choice   = 0;					# Choice of action


	#####################
	# Program area	
	

	# Primary security check
	# Password Validity
	if (CheckDataPassword($database, $param->{'USER'}, $param->{'OLDPASS'}))
	{ 
		# Increment by one
		$choice++; 
	}
	else
	{ 
		# Display error message
		print "$config->{ErrPassword}\n"; 
	}


	# Password change
	if (defined($param->{'ONEPASS'}))
	{
		# Valid new passwords
		if ((CheckDataLenght($param->{'ONEPASS'})) && ($param->{'ONEPASS'} eq $param->{'TWOPASS'}) )
		{
			# Inrement by one 
			$choice++; 
		}
		else
		{
			# Display error message 
			print "$config->{ErrMismatch}\n"; 
		}
	}	

	# E-Mail change
	elsif (defined($param->{'EMAIL'}))
	{
		# Valid new E-Mail
		if (CheckDataEMail($param->{'EMAIL'}))
		{	
			# Inrement by one 
			$choice++; 
		}
		else
		{ 
			# Display error message
			print "$config->{ErrEMail}\n"; 
		}
	}

	# If both tests pass, change data
	if ($choice == 2)
	{
		# Password change
		if ( defined($param->{'ONEPASS'}) )
		{
			# Create random string
			my $rand = md5_hex(rand());
			$rand = substr($rand, 0, 14);			

			# Insert new password
			$data->do("UPDATE Puppeteer 
				SET   PuppeteerPassword=? 
				WHERE PuppeteerLogin=?", {}, 

			 crypt($param->{'ONEPASS'}, $rand), 
		 	 $param->{'USER'});
		}
		
		# EMail change
		elsif ( defined($param->{'EMAIL'}) )
		{
			# Insert new password
			$data->do("UPDATE Puppeteer 
				SET PuppeteerEMail=? 
				WHERE PuppeteerLogin=?", {}, 
			 $param->{'EMAIL'}, 
			 $param->{'USER'});
		}

		# Notify of Success
		print "$config->{TagConfSuccess}\n";
	}
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash
	my $login    = shift;					# Login hash

	my $table    = new Ethereal::Table($cgi);
	my $url      = $cgi->url();				# Self referencing URL

	my $active;						# Active handles
	my $inactive;						# Inactive handles

	my @active;						# Active handles
	my @inactive;						# Inactive handles
	
	my %login;						# Login hash
	

	#####################
	# Program area	
	
	# Link with hashes
	$database->GetHashLogin(\%login);

	# Generate Inlines
	my ($iactv) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});
	my ($imail) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'CMAIL', 'True');
	my ($ipass) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'PASSWORD', 'True');
	my ($ipref) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'PREFS', 'True');

	
	# Create Inline links
	print "<CENTER><A HREF=\"$url$iactv\">$config->{TagSuspend}</A>\n",
	      "<A HREF=\"$url$ipref\">$config->{TagPrefs}</A>\n",
	      "<A HREF=\"$url$imail\">$config->{TagEmail}</A>\n",
	      "<A HREF=\"$url$ipass\">$config->{TagPassword}</A></CENTER>\n<BR><BR>\n";

	# Password change
	if (defined($param->{'PASSWORD'}))
	{
		# Display of information
		# Title
		print "$config->{TagConfPassword}";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'PASSWORD', 'True'), "\n";

		# Password section
		$table->MakeTop();
		$table->MakeValid($login{'TagPasswordOld'}, $cgi->password_field('OLDPASS', '', 10, 8));
		$table->MakeValid($login{'TagPassword'}, $cgi->password_field('ONEPASS', '', 10, 8));
		$table->MakeValid($login{'TagPasswordRepeat'}, $cgi->password_field('TWOPASS', '', 10, 8));
		$table->MakeBlank();
	
		# Submit and bottom
		$table->MakeValid($cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}


	# Email address change
	elsif (defined($param->{'CMAIL'}))
	{
		# Display of information
		# Title
		print "$config->{TagConfEMail}\n";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'CMAIL', 'True');


		# Email section
		$table->MakeTop();
		$table->MakeValid($login{'TagPassword'}, $cgi->password_field('OLDPASS', '', 10, 8));
		$table->MakeValid($login{'TagEMail'}, $cgi->textfield('EMAIL', '', 20, 45));
		$table->MakeBlank();

		# Submit and bottom
		$table->MakeValid($cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}

	# Chat prefernces
	elsif (defined($param->{'PREFS'}))
	{
		my $option;					# Option handler
		my $rjava;					# Formated javascript
		my $rchat;					# Formatted chat preferences
		my $rformat;					# Formatted format options

		my @rjava;					# Javascript options
		my @rchat;					# Chat options
		my @rformat;					# Formatting options


		# Split data into arrays
		$option = new Ethereal::Option();
		$option->Split(\@rjava, $login{'OptJavascript'});
		$option->Split(\@rchat, $login{'OptChatPref'});
		$option->Split(\@rformat, $login{'OptFormat'});

		# Data check
		$param->{'CJAVA'}   = defined($param->{'CJAVA'})   ? $param->{'CJAVA'}   : $rjava[2];
		$param->{'CCHAT'}   = defined($param->{'CCHAT'})   ? $param->{'CCHAT'}   : $rchat[4];
		$param->{'CFORMAT'} = defined($param->{'CFORMAT'}) ? $param->{'CFORMAT'} : $rformat[3];


		# Format options nicely
		$rjava   = $cgi->radio_group('CJAVA',   \@rjava,   $param->{'CJAVA'},   'true');
		$rchat   = $cgi->radio_group('CCHAT',   \@rchat,   $param->{'CCHAT'},   'true');
		$rformat = $cgi->radio_group('CFORMAT', \@rformat, $param->{'CFORMAT'}, 'true');


		# Display of information
		# Title
		print "$config->{TagConfPrefs}\n";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param});

		# Display of options
		$table->MakeTop();

		# Javascript
		$table->MakeSingle($login->{'TagOptJavacript'});
		$table->MakeSingle($rjava);
		$table->MakeBlank();

		# Chat options
		$table->MakeSingle($login->{'TagOptChatPref'});
		$table->MakeSingle($rchat);
		$table->MakeBlank();

		# Formating options
		$table->MakeSingle($login->{'TagOptFormat'});
		$table->MakeSingle($rformat);
		$table->MakeBlank();
		
		# Submit and bottom
		$table->MakeSingle($cgi->submit($login{'TxtSubmitButtonSub'}));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}



	# Puppet Ignore/Suspend
	else
	{
		# Populate arrays
		$database->GetListPuppetYours(\@active, $param->{'USER'});
		$database->GetListPuppetSuspended(\@inactive, $param->{'USER'});

		# Format lists
		$active   = $cgi->scrolling_list('APUPPET', \@active,   $active[0],   5);
		$inactive = $cgi->scrolling_list('IPUPPET', \@inactive, $inactive[0], 5);

		# Display of information
		# Title
		print "$config->{TagConfActivate}\n";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'PUPPET', 'True');

		# Activation section
		$table->MakeTop();
		$table->MakeSingle($config->{'TagActive'});
		$table->MakeValid($active,   $cgi->submit($config->{'TxtInactive'}));
		$table->MakeBlank();
		$table->MakeSingle($config->{'TagInactive'});
		$table->MakeValid($inactive, $cgi->submit($config->{'TxtActive'}));

		# Bottom of page
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}
}
